@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -173,8 +173,21 @@ decode_token (const char **in)
 	decode_lwsp (&inptr);
 	
 	start = inptr;
+#ifdef STRICT_PARSER
 	while (is_ttoken (*inptr))
 		inptr++;
+#else
+	/* Broken mail clients like to make our lives difficult. Scan
+	 * for a ';' instead of trusting that the client followed the
+	 * specification. */
+	while (*inptr && *inptr != ';')
+		inptr++;
+	
+	/* Scan backwards over any trailing lwsp */
+	while (inptr > start && is_lwsp (inptr[-1]))
+		inptr--;
+#endif
+	
 	if (inptr > start) {
 		*in = inptr;
 		return g_strndup (start, (size_t) (inptr - start));
@@ -189,6 +202,7 @@ decode_value (const char **in)
 	const char *inptr = *in;
 	
 	decode_lwsp (&inptr);
+	*in = inptr;
 	
 	if (*inptr == '"') {
 		return decode_quoted_string (in);
@@ -196,7 +210,11 @@ decode_value (const char **in)
 		return decode_token (in);
 	}
 	
+#ifndef STRICT_PARSER
+	return decode_token (in);
+#else
 	return NULL;
+#endif
 }
 
 /* This function is basically the same as decode_token()
